%function to take immunohistochemistry images and segment them into
%distinctive high and low intensity regions, the inputs will from already
%images that have been deconvoluted into the brown channel only (tested
%using the QuPath deconvolution

%inputs:
%ihc file - the deconvoluted IHC image as any format readable using the imread function
%outputFolder - location to save the output figures to
%outputFilename - base filename to save the figure as
%scalingFactor - number of pixel downbinning to use

%the outputs are the images saved as both .fig and .tiff format

%Author - Alex Dexter
%Version 1.0
function [] = ihcHighLowClustering( ihcFile, outputFolder, outputFilename, scalingFactor )

%read the IHC file
io = imread(ihcFile, 'Index', 1);

%strip out background of IHC image by thresholding the blue channel at 230
backgroundRemovedIHC = io;
%get blue channel
blueChannel = backgroundRemovedIHC(:,:,2);
for k = 1:3
    %get current red green or blue
    temp = backgroundRemovedIHC(:,:,k);
    %set all pixels where blue channel >230 to be 0
    temp(blueChannel>230) = 0;
    %add this back into the image
    backgroundRemovedIHC(:,:,k) = temp;
end
%display the image and save
h = figure('units','normalized','outerposition',[0 0 1 1]);
imagesc(backgroundRemovedIHC)
axis image
axis off
saveas(h, [outputFolder filesep outputFilename 'thesholdedImage'], 'fig')
saveas(h, [outputFolder filesep outputFilename 'thesholdedImage'], 'tiff')
close(h)

%get a mask of tissue
mask = sum(backgroundRemovedIHC,3)~=0;
%set all remaining regions dentified as not background to be 1
ihcGradingImage = double(mask);
%set up a red and green colourscheme for high and low
cmap = [0 0 0; 1 0 0; 0 1 0];

%% Downbinning of grading image
%down bin the IHC data by the specified factor
resizedIHCgrading  = blockproc( ihcGradingImage, [scalingFactor scalingFactor], @(b) sum( b.data(:) ) );
%cluster the downbinned dataset into three clusters (for high low and background)
[im, C] = kmeans(reshape(resizedIHCgrading,[],1), 3);
%sort clusters by intensity for consistent colouring
[~, l] = sort(C);
%rearrange cluster numbers in order of mean intensity
ihcGradingDownsampled = im;
for i = 1:3
    ihcGradingDownsampled(im==l(i)) =i;
end
%make image of the downbinned clusters
h = figure;
imagesc(reshape(ihcGradingDownsampled, size(resizedIHCgrading,1), size(resizedIHCgrading,2)))
colormap(cmap)
axis image
axis off
colorbar
%save the image
saveas(h, [outputFolder filesep outputFilename 'IHCclustered'], 'fig')
saveas(h, [outputFolder filesep outputFilename 'IHCclustered'], 'tiff')
close(h)
%make the binary image of just IHC high regions
h = figure;
imagesc(reshape(ihcGradingDownsampled==3, size(resizedIHCgrading,1), size(resizedIHCgrading,2)))
colormap gray
axis image
axis off
%save the image
saveas(h, [outputFolder filesep outputFilename 'IHChighBinary'], 'fig')
saveas(h, [outputFolder filesep outputFilename 'IHChighBinary'], 'tiff')
close(h)
%make the image of the downbined high pixel proportions
h = figure;
imagesc(resizedIHCgrading./scalingFactor^2 * 50)
colormap magma
axis image
axis off
colorbar
%save the image
saveas(h, [outputFolder filesep outputFilename 'IHChighPercentage'], 'fig')
saveas(h, [outputFolder filesep outputFilename 'IHChighPercentage'], 'tiff')
close(h)

end

