# Pathway enrichment analysis using mummichod based analysis for WMHigh and WMLow enriched metabolites
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Avinash Ghanate (The Francis Crick Institute, London)
#

library(stringr)
library(ggthemes)
library(tidyverse)


# Read input files --------------------------------------------------------

# setwd("")  # Set the working directory here

## Read annotated compound list from Pathway enrichment analysis with fold change across sample groups from LCMS data
annotData <- read.csv("mummichog_matched_compound_all_mergedFC.csv")
## Pathway enrichment results from mummichog analysis 
pathwayDb <- read.csv("mummichog_pathway_enrichment.csv", stringsAsFactors=FALSE)


# Prepare data and calculate enrichment score -----------------------------

##  Identify enrichment score for WMHigh (green) or WMLow (red) samples as described in comments below
pathwayDb.upd <- pathwayDb %>% 
  mutate(EC.list=str_split(EC.Hits,pattern=";")) %>% 
  # Identify median FC if multiple EC with adducts combination found for same metabolite
  mutate(EC.FC=map(EC.list, ~map(.x, ~median(as.numeric(annotData$Fold.change.in.Red.Vs.Green[annotData$Empirical.Compound %in% .x]))))) %>%  
  # How many enriched in Green (FC<1)
  mutate(Hits.Sig.Green=unlist(map(EC.FC, ~length(which(.x<1))))) %>%
  # If more than 50% mets from Significant hits are enriched in Green based on FC, enrichment score calculated as
  #     ES=Hits.Sig.Green/Expected    or
  #     ES=-(Hits.sig - Hits.Sig.Green)/Expected (Negative sign for divergent bar plot)
  mutate(ES.GreenvsRed=if_else((Hits.Sig.Green/Hits.sig)>0.5, Hits.Sig.Green/Expected, -(Hits.sig - Hits.Sig.Green)/Expected)) %>%
  # Weighing ES using P-value
  mutate(ES.GreenvsRed=-1*ES.GreenvsRed*log10(FET)) %>% 
  mutate(ES.class=ifelse(ES.GreenvsRed>1, "Green", "Red")) %>% 
  # Get stat stars
  mutate(stat.signif=case_when(
    FET < 0.0001 ~ "****",
    FET < 0.001 ~ "***",
    FET < 0.01 ~ "**",
    FET < 0.05 ~ "*",
    TRUE ~ ""
  )) %>% 
  arrange(ES.GreenvsRed) %>% 
  mutate(X=factor(X, levels=X)) %>% 
  # Select top 5 pathways from both Green and Red
  slice(-(6:(n()-5))) %>% 
  select(-c(EC.list, EC.FC))

## Export the processed data of top pathways enriched in each sample types
write.csv(pathwayDb.upd, file = "mummichog_pathway_enrichment_perClass.csv")



# Save the enrichment bar plot --------------------------------------------

ggplot(pathwayDb.upd, aes(ES.GreenvsRed, X, fill=ES.class, label=stat.signif)) + 
  geom_bar(stat="identity") +
  scale_fill_manual(values=c("turquoise3", "tomato2")) + 
  geom_text(aes(x=ES.GreenvsRed*1.2), size=8) +
  theme_clean(base_size=20) +
  labs(x="Enrichment score", y="") +
  theme(legend.title=element_blank())
ggsave("Enriched-pathways.pdf", width=10, height=10)  
