# Binarised image for the labelled ions from infusion studies 
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Avinash Ghanate (The Francis Crick Institute, London)
#

options(warn = -1)
rm(list = ls())
library(EBImage)
library(sva)
library(hdf5r)
library(tidyverse)
library(rhdf5)
library(scales)
library(tidyverse)
library(viridisLite)
library(igraph)
library(ggraph)

# Define which analysis folder to use -------------------------------------

ANALYSIS <- "glucose" # It can be 'glucose' or 'glutamine' or 'pdx'
# setwd("")  # Set the working directory here

source("./functions.R")

if (ANALYSIS == "glucose") {
  data_dir <- "./DATA/glucose_infused"
} else if (ANALYSIS == "glutamine") {
  data_dir <- "./DATA/glutamine_infused"
} else if (ANALYSIS == "pdx") {
  data_dir <- "DATA/pdx_infused"
}

plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)



# List one directory with MSI data for binary image -----------------------

fileN <- listMSIDirs(data_dir)
# # To select single scan
# if (ANALYSIS == "pdx"){
#   fileN <- fileN[1]
# } else if(length(fileN)>1) {
#   fileN <- fileN[2]
# }


# Read data and prepare it for plotting -----------------------------------

totalData <- tibble(fileN) %>% 
  mutate(fileIndx = seq_len(n())) %>% 
  mutate(h5fc = map(fileN, ~h5file(paste0(.x,"/intensity_mat_multi.h5"), mode = "r"))) %>% 
  mutate(mz = map(h5fc, ~.x[["cmz"]]$read()),
         dim_xy = map(h5fc, ~.x[["dim_xy"]]$read()),
         allData = map(h5fc, ~.x[["X"]]),
         tissue_idx = map(allData, ~which(colSums(.x$read())!=0)),
         data = map2(allData, tissue_idx, ~.x$read() %>% t() %>% as.data.frame() %>% slice(.y)),
         all_pixels = pmap(list(dim_xy, fileIndx, fileN), function(a,b,c) which(matrix(1, nrow=a[1], ncol = a[2]) == 1, T) %>% as.data.frame() %>% rename(x = row, y = col) %>% mutate(fileIndx = b, fileN = c))
  ) %>% 
  mutate(
    indx = map2(all_pixels, tissue_idx, ~.x %>% as.data.frame() %>% slice(.y)))

outData <- totalData %>% 
  # Comment this if you dont want normalization
  mutate(data = map(data, ~normData(.x, norm = "median") %>% as.data.frame())) %>%
  summarise(data = list(bind_rows(data)), indx = list(bind_rows(indx)), mz = list(mz[[1]]))
outData <- list(data = outData$data[[1]], mz = outData$mz[[1]], indx = outData$indx[[1]])


# Plot Pantothenic acid
plotMSI(outData$data, mzlist = outData$mz, indx = outData$indx, mz = 218.10340, ppm = 15, Title = "Pantothenic acid", norm = "max", revBG = FALSE) +
  facet_wrap(vars(fileIndx), ncol = 1, strip.position = "left") + theme(legend.position = "right")
ggsave(file.path(plots_dir, "Pantothenic acid_GlcInfused.pdf"))

# Ion list for plotting binary images -------------------------------------

mz <- c()
mz[1] <- 146.04588+5*1.003355 # Glutamate M+5
mz[2] <- 89.0244+ 3*1.003355 # Lactate M+3
mz[3] <- 218.10340 # Pantothenic acid
mz[4] <- 146.04588 # Glutamic acid
mz[5] <- 89.0244 # Lactic acid
mz[6] <- 133.0142+1*1.003355 # Malate
mz[7] <- 133.0142 # Malic acid
mz[8] <- 146.04588+2*1.003355 # Glutamate M+2


ppm <- 15
mzlist <- outData$mz
mzNum <- length(mzlist)

# Get m/z index for requested metabolic ion m/z
mzI <- matrix(NA, nrow = length(mz), ncol = 1)
for(i in 1:length(mz))
{    
  tmp_mzI <-
    which(abs(mzlist - mz[i]) * 1e6 < mz[i] * ppm)         ## within PPM window search
  
  if (length(tmp_mzI) > 1)
    tmp_mzI = tmp_mzI[which.min(abs(mz[i] - mzlist[tmp_mzI]))]        ## Find m/z with lowest PPM difference
  
  if(length(tmp_mzI) > 0)
    mzI[i] <- tmp_mzI
}

if (sum(unlist(lapply(mzI,length))) == 0)
  return()

gplot = ggplot() +
  labs(x = '', y = '', fill = '') +
  scale_y_reverse() +
  coord_equal() +
  theme_void() +
  theme(
    plot.subtitle = element_text(
      size = 7,
      hjust = 0,
      face = "italic",
      color = "black"
    ),
    panel.grid.major = element_blank(),
    axis.line = element_blank(),
    legend.position = "none"
  ) 



# Natural Abundance threshold plot for Lac M+3/Lac M+0 --------------------

#### Regions with intensity > natural abundance
# Highlight regions where Lac M+3 intensity is higher than its expected natural abundance based on,
# nat abun. for Lac M+3: Relative height to M+0 >= 0.0206 %
# 

fillInt <- ((outData$data[,mzI[[2]]]/outData$data[,mzI[[5]]])>0.000206)
fillInt[!fillInt] <- NA
natAbunChkLac <- fillInt
gplot +
  geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
  labs(title="Lac M+3", fill="Lac M+3 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
if (ANALYSIS == "glucose") {
  ggsave(file.path(plots_dir, "LacM3NatAbundCheck_GlcInfused.pdf"))
} else {
  ggsave(file.path(plots_dir, "LacM3NatAbundCheck_GlnInfused.pdf"))
}


if(ANALYSIS != "pdx") {
  # Natural Abundance threshold plot for Mal M+1/Mal M+0 --------------------
  
  #### Regions with intensity > natural abundance
  # Highlight regions where Mal M+1 intensity is higher than its expected natural abundance based on,
  # nat abun. for Mal M+1: Relative height to M+0 >= 4.6068 %  
  # 95% quantile for intensity ratio values of Mal M+1/Mal M+0 in non-infused data will be used as natural abundance cut-off instead. 
  # Rest other metabolites and their heavier isotopologues of interest were not found in non-infused so were treated using known natural abundance cut-off criteria
  # 
  
  fillInt <- ((outData$data[,mzI[[6]]]/outData$data[,mzI[[7]]])>0.0921)
  fillInt[!fillInt] <- NA
  natAbunChkMal <- fillInt
  gplot +
    geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
    labs(title="Mal M+1", fill="Mal M+1 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
  if (ANALYSIS == "glucose") {
    ggsave(file.path(plots_dir, "MalM1NatAbundCheck_GlcInfused.pdf"))
  } else {
    ggsave(file.path(plots_dir, "MalM1NatAbundCheck_GlnInfused.pdf"))
  }
  
  
  
  # Overlaid binary plots for Mal M+1 and Lac M+3 ---------------------------
  
  malResp <- data.frame(int = outData$data[,mzI[[6]]], indx = outData$indx$fileIndx) %>% 
              group_nest(indx) %>% 
              mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
              pull(data) %>% 
              bind_rows() %>% 
              pull(int)
  lacResp <- data.frame(int = outData$data[,mzI[[2]]], indx = outData$indx$fileIndx) %>% 
              group_nest(indx) %>% 
              mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
              pull(data) %>% 
              bind_rows() %>% 
              pull(int)
  
  fillIntDiff <- (malResp - lacResp)
  fillInt <- ifelse(fillIntDiff!=0,ifelse(fillIntDiff>0,"Mal M+1", "Lac M+3"), "NA") # Set to NA if difference is 0 or pos difference is Mal rich and vice versa
  
  # To remove pixels where it does not meet natural abundance criteria
  fillInt[which(is.na(natAbunChkMal) & (fillInt == 1))] <- "NA"
  fillInt[which(is.na(natAbunChkLac) & (fillInt == 0))] <- "NA"
  
  
  gplot +
    geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), interpolate = T) +
    scale_fill_manual(values = c("indianred1", "springgreen1", "grey86")) +
    labs(title="Mal M+1 (Green) vs Lac M+3 (Red)") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
  if (ANALYSIS == "glucose") {
    ggsave(file.path(plots_dir, "MalM1LacM3_GlcInfused.pdf"))
  } else {
    ggsave(file.path(plots_dir, "MalM1LacM3_GlnInfused.pdf"))
  }
  
  if (ANALYSIS == "glucose") {
    # Natural Abundance threshold plot for Glu M+2/Glu M+0 --------------------
    
    #### Regions with intensity > natural abundance
    # Highlight regions where Glu M+2 intensity is higher than its expected natural abundance based on,
    # nat abun. for Glu M+2: Relative height to M+0 >= 0.9555 % 
    
    fillInt <- ((outData$data[,mzI[[8]]]/outData$data[,mzI[[4]]])>0.009555)
    fillInt[!fillInt] <- NA
    natAbunChkGlu <- fillInt
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
      labs(title="Glu M+2", fill="Glu M+2 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    
    ggsave(file.path(plots_dir, "GluM2NatAbundCheck_GlcInfused.pdf"))
    
    
    # Overlaid binary plots for Glu M+2 and Lac M+3 ---------------------------
    
    gluResp <- data.frame(int = outData$data[,mzI[[8]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    lacResp <- data.frame(int = outData$data[,mzI[[2]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    fillIntDiff <- (gluResp - lacResp)
    fillInt <- ifelse(fillIntDiff!=0,ifelse(fillIntDiff>0,"Glu M+2", "Lac M+3"), "NA") # Set to NA if difference is 0 or pos difference is Glu rich and vice versa
    
    # To remove pixels where it does not meet natural abundance criteria
    fillInt[which(is.na(natAbunChkGlu) & (fillInt == 1))] <- "NA"
    fillInt[which(is.na(natAbunChkLac) & (fillInt == 0))] <- "NA"
    
    
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), interpolate = T) +
      scale_fill_manual(values = c("springgreen1","indianred1", "grey86")) +
      labs(title="Glu M+2 (Green) vs Lac M+3 (Red)") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    ggsave(file.path(plots_dir, "GluM2LacM3_GlcInfused.pdf"))
  } else {
    # Natural Abundance threshold plot for Glu M+2/Glu M+0 --------------------
    
    #### Regions with intensity > natural abundance
    # Highlight regions where Glu M+2 intensity is higher than its expected natural abundance based on,
    # nat abun. for Glu M+2: Relative height to M+0 >= 0.9555 % 
    
    fillInt <- ((outData$data[,mzI[[8]]]/outData$data[,mzI[[4]]])>0.009555)
    fillInt[!fillInt] <- NA
    natAbunChkGlu <- fillInt
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
      labs(title="Glu M+2", fill="Glu M+2 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    
    ggsave(file.path(plots_dir, "GluM2NatAbundCheck_GlnInfused.pdf"))
    
    
    # Overlaid binary plots for Glu M+2 and Lac M+3 ---------------------------
    
    gluResp <- data.frame(int = outData$data[,mzI[[8]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    lacResp <- data.frame(int = outData$data[,mzI[[2]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    fillIntDiff <- (gluResp - lacResp)
    fillInt <- ifelse(fillIntDiff!=0,ifelse(fillIntDiff>0,"Glu M+2", "Lac M+3"), "NA") # Set to NA if difference is 0 or pos difference is Glu rich and vice versa
    
    # To remove pixels where it does not meet natural abundance criteria
    fillInt[which(is.na(natAbunChkGlu) & (fillInt == 1))] <- "NA"
    fillInt[which(is.na(natAbunChkLac) & (fillInt == 0))] <- "NA"
    
    
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), interpolate = T) +
      scale_fill_manual(values = c("springgreen1","indianred1", "grey86")) +
      labs(title="Glu M+2 (Green) vs Lac M+3 (Red)") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    ggsave(file.path(plots_dir, "GluM2LacM3_GlnInfused.pdf"))
  
    # Natural Abundance threshold plot for Glu M+5/Glu M+0 --------------------
    
    #### Regions with intensity > natural abundance
    # Highlight regions where Glu M+5 intensity is higher than its expected natural abundance based on,
    # nat abun. for Glu M+5: Relative height to M+0 >= 0.0001 % 
    
    fillInt <- ((outData$data[,mzI[[1]]]/outData$data[,mzI[[4]]])>0.000001)
    fillInt[!fillInt] <- NA
    natAbunChkGlu <- fillInt
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
      labs(title="Glu M+5", fill="Glu M+5 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    ggsave(file.path(plots_dir, "GluM5NatAbundCheck_GlnInfused.pdf"))
    
    
    
    # Overlaid binary plots for Glu M+5 and Lac M+3 ---------------------------
    
    gluResp <- data.frame(int = outData$data[,mzI[[1]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    lacResp <- data.frame(int = outData$data[,mzI[[2]]], indx = outData$indx$fileIndx) %>% 
      group_nest(indx) %>% 
      mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
      pull(data) %>% 
      bind_rows() %>% 
      pull(int)
    
    fillIntDiff <- (gluResp - lacResp)
    fillInt <- ifelse(fillIntDiff!=0,ifelse(fillIntDiff>0,"Glu M+5", "Lac M+3"), "NA") # Set to NA if difference is 0 or pos difference is Glu rich and vice versa
    
    # To remove pixels where it does not meet natural abundance criteria
    fillInt[which(is.na(natAbunChkGlu) & (fillInt == 1))] <- "NA"
    fillInt[which(is.na(natAbunChkLac) & (fillInt == 0))] <- "NA"
    
    
    gplot +
      geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), interpolate = T) +
      scale_fill_manual(values = c("springgreen1","indianred1", "grey86")) +
      labs(title="Glu M+5 (Green) vs Lac M+3 (Red)") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
    ggsave(file.path(plots_dir, "GluM5LacM3_GlnInfused.pdf"))
  }

} else if(ANALYSIS == "pdx") {
  # Natural Abundance threshold plot for Glu M+5/Glu M+0 --------------------
  
  #### Regions with intensity > natural abundance
  # Highlight regions where Glu M+5 intensity is higher than its expected natural abundance based on,
  # nat abun. for Glu M+5: Relative height to M+0 >= 0.0001 % 
  
  fillInt <- ((outData$data[,mzI[[1]]]/outData$data[,mzI[[4]]])>0.000001)
  fillInt[!fillInt] <- NA
  natAbunChkGlu <- fillInt
  gplot +
    geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), na.rm = TRUE) +
    labs(title="Glu M+5", fill="Glu M+5 intensity \nhigher than natural abundance") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
  ggsave(file.path(plots_dir, "GluM5NatAbundCheck_GlnInfused.pdf"))
  
  
  
  # Overlaid binary plots for Glu M+5 and Lac M+3 ---------------------------
  
  gluResp <- data.frame(int = outData$data[,mzI[[1]]], indx = outData$indx$fileIndx) %>% 
              group_nest(indx) %>% 
              mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
              pull(data) %>% 
              bind_rows() %>% 
              pull(int)
    
  lacResp <- data.frame(int = outData$data[,mzI[[2]]], indx = outData$indx$fileIndx) %>% 
              group_nest(indx) %>% 
              mutate(data = map(data, ~ .x/max(.x, na.rm = T))) %>% 
              pull(data) %>% 
              bind_rows() %>% 
              pull(int)
  
  fillIntDiff <- (gluResp - lacResp)
  fillInt <- ifelse(fillIntDiff!=0,ifelse(fillIntDiff>0,"Glu M+5", "Lac M+3"), "NA") # Set to NA if difference is 0 or pos difference is Glu rich and vice versa
  
  # To remove pixels where it does not meet natural abundance criteria
  fillInt[which(is.na(natAbunChkGlu) & (fillInt == 1))] <- "NA"
  fillInt[which(is.na(natAbunChkLac) & (fillInt == 0))] <- "NA"
  
  
  gplot +
    geom_raster(data = outData$indx, aes(x = x, y = y, fill = fillInt), interpolate = T) +
    scale_fill_manual(values = c("springgreen1","indianred1", "grey86")) +
    labs(title="Glu M+5 (Green) vs Lac M+3 (Red)") + facet_wrap(vars(fileIndx), ncol = 1) + theme(legend.position = "right")
  ggsave(file.path(plots_dir, "GluM5LacM3_GlnInfused.pdf"))
}
