# Plot Single Ion Images for metabolic ions of interest and generate dendrogram
#
# MIT License
# 
# Copyright (c) 2021 Paolo Inglese
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Paolo Inglese (Imperial College London)
#

library(rhdf5)
library(EBImage)
library(lattice)
library(gridExtra)
library(rasterVis)
library(WGCNA)

ANALYSIS <- "glucose" # It can be 'glucose' or 'glutamine' or 'pdx'

# setwd("")  # Set the working directory here

source("functions.R")

proxy <- read.csv("search_metab.csv")

adduct <- list(H = -1.007276)
delta <- 1.00335

if (ANALYSIS == "glucose") {
  data_dir <- "DATA/glucose_infused"
} else if (ANALYSIS == "glutamine") {
  data_dir <- "DATA/glutamine_infused"
} else if (ANALYSIS == "pdx") {
  data_dir <- "DATA/pdx_infused"
}
rds_dir <- file.path(data_dir, "_RDS")
if (!dir.exists(rds_dir)) dir.create(rds_dir, recursive = T)
plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)
csv_dir <- file.path(data_dir, "_CSV")
if (!dir.exists(csv_dir)) dir.create(csv_dir, recursive = T)

msi_dirs <- listMSIDirs(data_dir)
n_runs <- length(msi_dirs)
print(paste0("Num. runs = ", n_runs))

sel_ions <- list(
  glucose = list(
    c("Lactic acid", 3),
    c("Fumaric acid", 1),
    c("Malic acid", 1),
    c("L-Aspartic Acid", 1),
    c("L-Glutamate", 1),
    c("L-Glutamate", 2),
    c("L-Glutamate", 3),
    c("Pantothenic Acid", 0)
  ),
  glutamine = list(
    c("Lactic acid", 3),
    c("Fumaric acid", 1),
    c("Malic acid", 1),
    c("L-Aspartic Acid", 1),
    c("L-Glutamate", 1),
    c("L-Glutamate", 3),
    c("L-Glutamate", 5),
    # c('L-Glutamine', 1),
    c("L-Glutamine", 3),
    c("L-Glutamine", 5),
    c("Pantothenic Acid", 0)
  ),
  pdx = list(
    c("Lactic acid", 3),
    c("L-Glutamate", 5),
    c("Pantothenic Acid", 0)
  )
)

# Load -------------------------------------------------------------------------

cmz <- rhdf5::h5read(file.path(msi_dirs[1], "intensity_mat_multi.h5"), "cmz")

all_X <- c()
for (i in 1:n_runs) {
  X <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "X")
  X <- t(X)
  non_zero <- apply(X, 1, function(z) any(z != 0))
  all_X <- rbind(all_X, X[non_zero, ])
  rm(X, non_zero)
}

matches <- searchMetab(proxy, cmz, all_X, adduct$H, delta, max_err = 15)
write.csv(matches, file = file.path(csv_dir, "match_metab_H.csv"))

for (i in 1:n_runs) {
  run_id <- strsplit(msi_dirs[i], "/")[[1]][-1]

  cmz <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "cmz")
  dxy <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "dim_xy")
  X <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "X")

  h5closeAll()

  dxy <- dxy[2:1]
  X <- t(X)

  print(matches)

  non_zero <- apply(X, 1, function(x) any(x != 0))
  im_nz <- matrix(non_zero, dxy[2:1])

  # Correct labelled isotopes
  X <- correctLabelled(X, matches)

  # Test ions of interest --------------------------------------------------------

  plot_dir <- paste(c(plots_dir, "metabolites_H", run_id[length(run_id)]), collapse = "/")
  if (!dir.exists(plot_dir)) {
    dir.create(plot_dir, recursive = T)
  }
  file.remove(list.files(plot_dir, full.names = T, recursive = F))

  metab_names <- unique(matches$name)
  mytheme <- infernoTheme()
  mytheme$panel.background$col <- "grey"

  for (n in metab_names) {
    tab_met <- subset(matches, name == n)
    print(tab_met)

    p <- vector("list", nrow(tab_met))
    for (i in 1:nrow(tab_met)) {
      main_str <- paste0(
        tab_met$name[i], " + ", tab_met$M[i], "\n",
        "(err=", round(tab_met$error[i], 2), "ppm)"
      )
      im <- array(NA, prod(dxy[2:1]))
      im[non_zero] <- X[non_zero, tab_met$idx[i]]
      cuts <- seq(
        min(im, na.rm = T), max(im, na.rm = T),
        (max(im, na.rm = T) - min(im, na.rm = T)) / 63
      )
      r <- raster::raster(t(matrix(im, dxy[2:1])),
        xmn = 1, xmx = dxy[2],
        ymn = 1, ymx = dxy[1]
      )
      p[[i]] <- rasterVis::levelplot(r,
        at = cuts, par.settings = mytheme,
        main = main_str
      )
    }

    if (nrow(tab_met) == 1) {
      nc <- 1
    } else {
      nc <- 2
    }
    nr <- ceiling(nrow(tab_met) / nc)

    png(
      filename = file.path(plot_dir, paste0(n, ".png")), res = 150,
      w = 8 * nc, h = 6 * nr, u = "in"
    )
    gridExtra::grid.arrange(grobs = p, nrow = nr, ncol = nc)
    dev.off()
  }
}

# Clusters selected metabolites

# Use only mix myc pixels
tissue_labels <- readRDS(file.path(rds_dir, "tissue_labels.rds"))

# Selected ions to cluster
sel_idx <- unlist(lapply(sel_ions[[ANALYSIS]], function(x) {
  subset(matches, (name == x[1]) & (M == x[2]))$idx
}))

X <- dxy <- vector("list", n_runs)
for (i in 1:n_runs) {
  cmz <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "cmz")
  dxy[[i]] <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "dim_xy")
  X[[i]] <- rhdf5::h5read(file.path(msi_dirs[i], "intensity_mat_multi.h5"), "X")
  X[[i]] <- t(X[[i]])
  dxy[[i]] <- dxy[[i]][2:1]
  if (ANALYSIS == "pdx") {
    X[[i]] = correctLabelled(X[[i]], matches)
    X[[i]] = list(data = X[[i]][, sel_idx])
  } else {
    X[[i]] <- list(data = X[[i]][which(tissue_labels[[i]] == "mix"), ])
    X[[i]]$data <- correctLabelled(X[[i]]$data, matches)[, sel_idx]
  }
  rhdf5::h5closeAll()
}

names_mol <- unlist(lapply(sel_ions[[ANALYSIS]], function(x) paste0(x, collapse = "+")))
rho <- array(NA, c(length(sel_idx), length(sel_idx), n_runs),
  dimnames = list(names_mol, names_mol, NULL)
)
for (i in 1:n_runs) {
  rho[, , i] <- cor(X[[i]]$data)
}

rho_cons <- matrix(NA, length(sel_idx), length(sel_idx),
  dimnames = list(names_mol, names_mol)
)
for (i in 1:dim(rho)[1]) {
  for (j in 1:dim(rho)[2]) {
    rho_cons[i, j] <- min(rho[i, j, ])
  }
}

dendro <- hclust(as.dist(1 - rho_cons), method = "average")
plot(dendro)
yclust <- cutree(dendro, k = 2)

png(
  filename = file.path(plots_dir, "dendrogram_metab.png"), res = 300,
  u = "in", w = 8, h = 6
)
WGCNA::plotDendroAndColors(dendro, WGCNA::labels2colors(yclust))
dev.off()
