# Heatmap for Myc signature correlation with Pantothenic acid related gene response
#
# MIT License
# 
# Copyright (c) 2021 Neil Patani
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Neil Patani (The Francis Crick Institute, London)
#

library(ComplexHeatmap)
library(data.table)
library(tidyverse)
library(circlize)

# Load input files --------------------------------------------------------

# setwd("")  # Set the working directory here
load("METABRIC_DATA.RData")
MYC_SIG_Order <- read.csv(file="MYC SIG ORDER.csv")
MYC_SIG_Order_Vector <- as.vector(MYC_SIG_Order$Row.names)
myc_sig <- read.csv(file="MYC Signature.csv", stringsAsFactors = F)



# Retrieve average Myc expression status ----------------------------------

MYC_SIG_COMP_CLEAR_Matrix <- METABRIC.data[match(myc_sig$MYC.SIGNATURE, METABRIC.data$external_gene_name), ] %>% 
  na.omit() %>% 
  remove_rownames() %>%
  column_to_rownames(var = 'external_gene_name')

MYC_SIG_COMP_AVE_DF <- colMeans(MYC_SIG_COMP_CLEAR_Matrix) %>% 
  as.data.frame() %>% 
  setDT(keep.rownames = TRUE)

names(MYC_SIG_COMP_AVE_DF) <- c("Row.names", "MYC_SIG_COMP_AVE")
names(Complete_METABRIC_Clinical_Features_Data)[names(Complete_METABRIC_Clinical_Features_Data) %in% c("ER_IHC_status","NOT_IN_OSLOVAL_Pam50Subtype")] <- c("ER status", "PAM50")


# Retrieve expression for genes involving Pantothenic acid transpo --------

include_list <- c("AURKA",  "SLC5A6", "PANK1",  "PANK2")

sel.data <- METABRIC.data[match(include_list, METABRIC.data$external_gene_name), ] %>% 
  remove_rownames() %>%
  column_to_rownames(var = 'external_gene_name') %>% 
  t() 



# Merge gene expression with the clinical features ------------------------

combined.data <- sel.data %>%
  as.data.frame() %>% 
  setDT(keep.rownames = "Row.names") %>% 
  left_join(MYC_SIG_COMP_AVE_DF) %>% 
  mutate(Row.names = gsub("\\.","-",Row.names)) %>% 
  left_join(Complete_METABRIC_Clinical_Features_Data, by = c("Row.names" = "sample")) %>% 
  mutate(PAM50 = gsub("LumA", "A_LumA", PAM50)) %>% 
  as.data.frame()


# Prepare plot options and plot heatmap -----------------------------------

col_fun = colorRamp2(c(-2,0,5), c("blue","white","red"))
col_fun2 = colorRamp2(c(1,3), c("white","blue"))
col_PAM = c("A_LumA" = "green", "LumB" = "blue", "Her2" = "purple", "Basal" = "red", "Normal" = "yellow", "NC" = "black")
col_ER = c("pos" = "green", "neg" = "red", "NC" = "white")
col_Hist = c("A_IDC" = "green", "ILC" = "red", "NC" = "white", "DCIS" = "white", "Other" = "white")
col_size = colorRamp2(c(0,190), c("white","black"))
col_LN = colorRamp2(c(0,45), c("white","black"))
col_NPI = colorRamp2(c(1,7), c("white","black"))
col_MYC_COMP = colorRamp2(c(-0.25,0,0.45), c("blue","white","red"))
col_PC1_COMP = colorRamp2(c(-16,0,25), c("blue","white","red"))

col_annot <- HeatmapAnnotation(df= combined.data[c("PAM50", "ER status", "grade", "size", "lymph_nodes_positive", "AURKA", "MYC_SIG_COMP_AVE")], col = list ("AURKA" = col_fun, "grade" = col_fun2, "PAM50" = col_PAM, "ER status" = col_ER, "size" = col_size, "lymph_nodes_positive" = col_LN, "MYC_SIG_COMP_AVE" = col_MYC_COMP), show_legend=TRUE)


pdf("Heatmap.pdf", width = 20, height = 3)
Heatmap(t(scale(sel.data[,-1],center = F)),
        top_annotation=col_annot, column_order = MYC_SIG_Order_Vector,
        show_row_names=TRUE, show_column_names=FALSE,
        show_heatmap_legend = TRUE)
dev.off()