# Calculate conditional probability for given Myc levels
#
# MIT License
# 
# Copyright (c) 2021 Paolo Inglese
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Paolo Inglese (Imperial College London)
#

library(bnlearn)
library(ComplexHeatmap)
library(RColorBrewer)
library(rhdf5)
library(EBImage)
library(sva)


# FUNCTIONS ----

loadMSI = function(dir) {  
  
  res = list(X = c(), cmz = c(), dxy = c())
  res$cmz = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'cmz')
  res$dxy = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'dim_xy')
  res$X = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'X')
  h5closeAll()
  
  res$dxy = res$dxy[2:1]
  res$X = t(res$X)
  res$cmz = sort(res$cmz)
  res$tissue_lbl = which(res$X != 0)
  
  return(res)
}

medNorm = function(x) {
  x[x == 0] = NA
  x = x / apply(x, 1, median, na.rm = T)
  x[is.na(x)] = 0
  return(x)
}

getROI = function(X, dxy, min_size = 10) {
  
  roi = apply(X, 1, function(x) any(x != 0))
  
  lbl_im = bwlabel(matrix(roi, dxy[2:1]))
  ul = unique(lbl_im[lbl_im != 0])
  areas = sapply(ul, function(x) sum(lbl_im == x))
  sel_ul = ul[areas > min_size]
  
  lbl_im = lbl_im %in% sel_ul
  return(matrix(lbl_im, dxy[2:1]))
}

fixBatch = function(msi_data) {
  
  n_runs = length(msi_data)
  
  # Merge data and correct batch effect
  x_all = c()
  batch = c()
  n_pix = c()
  for (i in 1:n_runs) {
    x_all = rbind(x_all, msi_data[[i]]$X)
    n_pix = c(n_pix, nrow(msi_data[[i]]$X))
    batch = c(batch, rep(i, nrow(msi_data[[i]]$X)))
  }
  
  x_all = t(sva::ComBat(t(x_all), batch = factor(batch)))
  
  offs = 0
  for (i in 1:n_runs) {
    msi_data[[i]]$X = x_all[(offs + 1):(offs + n_pix[i]), ]
    offs = offs + n_pix[i]
  }
  
  return(msi_data)
}


# START -------------------------------------------------------------------

## Define which analysis folder to use

# setwd("")  # Set the working directory here

data_dir <- "./DATA/no_infused"
plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)


# List all directories with MSI data --------------------------------------

rds_dir = file.path(data_dir, '_RDS')
plots_dir = file.path(data_dir, '_PLOTS')
csv_dir = file.path(data_dir, '_CSV')

msi_dirs = listMSIDirs(data_dir)
n_runs = length(msi_dirs)
print(paste0('Num. runs = ', n_runs))

msi_data = vector(mode = 'list', length = n_runs)
for (i in 1:n_runs) {
  msi_data[[i]] = loadMSI(msi_dirs[i])
  msi_data[[i]]$X = medNorm(msi_data[[i]]$X)
}

cmz = msi_data[[1]]$cmz  # common m/z values


# Select the ions of interest ---------------------------------------------

# Using Pathway set of ions
enrichPathway <- c("Porphyrin and chlorophyll metabolism", "Pyrimidine metabolism", "Pantothenate and CoA biosynthesis", 
                   "Glycine, serine and threonine metabolism", "Aminoacyl-tRNA biosynthesis")
pathwayRes <- read.csv(file.path("Kreuzaler_etal_Fig1b", "mummichog_matched_compound_all_mergedFC.csv"), stringsAsFactors = F)
pathwayRes <- pathwayRes %>% 
  filter(grepl(paste(enrichPathway, collapse = "|"), pathway)) %>% 
  filter(Matched.Form == "M-H[-]") %>% 
  filter(Empirical.Compound != "EC00051") %>% ## Removing less abundant ion
  distinct(Empirical.Compound, .keep_all = T) %>% 
  distinct(Matched.Compound, .keep_all = T) %>%
  mutate(mz = Query.Mass) %>% 
  arrange(mz)
  

# Get m/z index for requested metabolic ion m/z
mz = unique(pathwayRes$mz)
ppm = 10
sel_idx <- matrix(NA, nrow = length(mz), ncol = 1)
for(i in 1:length(mz))
{    
  tmp_mzI <-
    which(abs(cmz - mz[i]) * 1e6 < mz[i] * ppm)         ## within PPM window search
  
  if (length(tmp_mzI) > 1)
    tmp_mzI = tmp_mzI[which.min(abs(mz[i] - cmz[tmp_mzI]))]        ## Find m/z with lowest PPM difference
  
  if(length(tmp_mzI) > 0)
    sel_idx[i] <- tmp_mzI
}
sel_idx <-  sel_idx %>% unlist
rem_idx = is.na(sel_idx)
sel_idx <- sel_idx[!is.na(sel_idx)]
sel_mz <- cmz[sel_idx]


# Prepare MSI data for analysis -------------------------------------------

msi_data = lapply(msi_data, function(x) {
  x$X = x$X[, sel_idx]
  return(x)
})

# Tissue ROIs
rois <- lapply(1:length(msi_data), function(i) getROI(msi_data[[i]]$X, msi_data[[i]]$dxy))
tissue_labels <- readRDS(file.path(rds_dir, "tissue_labels.rds"))
for (i in 1:n_runs){
  tissue_labels[[i]][which(!rois[[i]])] = NA
}

# Batch effect correction - use only sample pixels
for (i in 1:n_runs){
  msi_data[[i]]$X = msi_data[[i]]$X[!is.na(tissue_labels[[i]]), ]
}
msi_data = fixBatch(msi_data)

filt_myc = lapply(tissue_labels, function(x) x[!is.na(x)])

# Concatenate the corrected intensities - use only lo/hi Myc
X_lo_hi_merge <- c()
myc_merge = c()
for (i in 1:n_runs)
{
  roi.px <- which(filt_myc[[i]] %in% c('lo', 'hi'))
  myc_merge = c(myc_merge, filt_myc[[i]][roi.px])
  X_lo_hi_merge <- rbind(
    X_lo_hi_merge,
    msi_data[[i]]$X[which(filt_myc[[i]] %in% c('lo', 'hi')), ])
  rm(roi.px)
}

colnames(X_lo_hi_merge) = round(sel_mz, 4)

# Discretize intensities - 3 levels

X_lo_hi_merge = bnlearn::discretize(as.data.frame(X_lo_hi_merge))
for (i in 1:ncol(X_lo_hi_merge)) {
  X_lo_hi_merge[, i] = factor(X_lo_hi_merge[, i])
  levels(X_lo_hi_merge[, i]) = c('lo', 'mid', 'hi')
}



# Calculate conditional probability ---------------------------------------

# P(feature | myc) - list

p_feat_myc_lo <- matrix(NA, length(sel_mz), 3, dimnames = list(round(sel_mz, 4), c('feat_lo', 'feat_mid', 'feat_hi')))
p_feat_myc_hi <- matrix(NA, length(sel_mz), 3, dimnames = list(round(sel_mz, 4), c('feat_lo', 'feat_mid', 'feat_hi')))
for (m in 1:length(sel_mz)) {
  feature_idx <- m
  subset_myc_lo <- which(myc_merge == 'lo')
  subset_myc_hi <- which(myc_merge == 'hi')
  for (f in c('lo', 'mid', 'hi')) {
    p_feat_myc_lo[paste0(round(sel_mz[m], 4)), paste0('feat_', f)] <- sum(X_lo_hi_merge[subset_myc_lo, feature_idx] == f) / length(subset_myc_lo)
    p_feat_myc_hi[paste0(round(sel_mz[m], 4)), paste0('feat_', f)] <- sum(X_lo_hi_merge[subset_myc_hi, feature_idx] == f) / length(subset_myc_hi)
  }
  rm(subset_myc_lo)
  rm(subset_myc_hi)
}



# Rearrange data for Heatmap and plot -------------------------------------

# Enriched pathway ions analysis
pathwayRes <- pathwayRes %>%
  slice(-which(rem_idx)) %>% 
  mutate(n = seq_len(n())) %>%
  mutate(p_feat_hi = p_feat_myc_hi[,3]) %>%
  arrange(desc(p_feat_hi), .by_group = T)

rownames(p_feat_myc_hi)[pathwayRes$n] <- pathwayRes %>% 
  separate(kegg.Cmpd, c("met","remMet"), sep = ";") %>% pull(met)
rownames(p_feat_myc_lo)[pathwayRes$n] <- pathwayRes %>% 
  separate(kegg.Cmpd, c("met","remMet"), sep = ";") %>% pull(met)


pdf(file.path(plots_dir, "Conditional probability Enriched Pathway.pdf"),
    width = 2.5, height = 6)
Heatmap(p_feat_myc_hi[pathwayRes$n,], cluster_rows = FALSE, cluster_columns = FALSE,
        row_names_gp = gpar(fontsize = 8), row_names_side = "left",
        name = "Probability", column_title = "Myc High")

Heatmap(p_feat_myc_lo[pathwayRes$n,], cluster_rows = FALSE, cluster_columns = FALSE,
        row_names_gp = gpar(fontsize = 8), row_names_side = "left",
        name = "Probability", column_title = "Myc Low")
dev.off()
