# Plot Single Ion Images for metabolic ions of interest
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Avinash Ghanate (The Francis Crick Institute, London)
#

options(warn = -1)
rm(list = ls())
library(tidyverse)


## Define which analysis folder to use -------------------------------------
# setwd("./Kreuzaler_etal_SupplFig2b_SupplFig2i-j/")  # Set the working directory here

source("../functions.R")
data_dir = "./input"
plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)

csv_dir <- file.path(data_dir, "_CSV")
if (!dir.exists(csv_dir)) dir.create(csv_dir, recursive = T)


# List all csv files with MSI data -----------------------------------------

fileN = list.files(data_dir, pattern = ".csv", full.names = T, recursive = F)


# Read data and prepare it for plotting -----------------------------------

totalData <- tibble(fileN) %>% 
  mutate(fileIndx = seq_len(n())) %>% 
  mutate(mz = gsub("./input/(\\d+.\\d+).*","\\1",fileN, perl = T) %>% as.numeric(),
  # mutate(mz = c(146.082, 218.101, 88.04),
         data = map(fileN, ~read.csv(.x) %>% as.matrix() %>% t()),
         dim_xy = map(data, ~dim(.x)),
         allData = map(data, ~as.vector(.x)),
         tissue_idx = map(allData, ~seq_along(.x)),
         all_pixels = map(dim_xy, ~which(matrix(1, nrow=.x[1], ncol = .x[2]) == 1, T) %>% as.data.frame() %>% rename(x = row, y = col) %>% mutate(fileIndx = 1, fileN = "1"))
  ) %>% 
  {
    tibble(
      allData = list(bind_cols(.$allData) %>% as.data.frame(col.names = as.character(.$mz))),
      dim_xy = list(.$dim_xy[[1]]),
      mz = list(.$mz),
      tissue_idx = list(.$tissue_idx %>% unlist() %>% unique()),
      all_pixels = list(.$all_pixels[[1]])
    )
  } %>%
  mutate(
    indx = map2(all_pixels, tissue_idx, ~.x %>% as.data.frame() %>% slice(.y))) %>% 
  mutate(
    data = map2(allData, tissue_idx, ~.x %>%  slice(.y)))


outData <- list(data = totalData$data[[1]], mz = totalData$mz[[1]], indx = totalData$indx[[1]])


# Plot single ion image for metabolite of interest (Suupl Fig 2b,i)------

ions <- tibble(
  mz = c(146.04533, 218.10285, 88.03985),
name=c("PA Fragment 1", "Pantothenic acid [M-H]-", "PA Fragment 2")
)

pdf(file.path(plots_dir, "Single ions images MSMS.pdf"),
    onefile = TRUE,
    width = 10)
  plotMSI(dataX = outData$data, indx = outData$indx,mzlist = outData$mz, mz = ions$mz[1], ppm = 300, Title = ions$name[1],
            revBG = F, norm = "max", remHotspot = T, lim.hotspot.int = 0.6) + facet_wrap(vars(fileIndx), ncol = 1) +
  labs(subtitle = paste("m/z:", outData$mz[[1]])) +
    scale_y_continuous()
  plotMSI(dataX = outData$data, indx = outData$indx,mzlist = outData$mz, mz = ions$mz[3], ppm = 300, Title = ions$name[3],
          revBG = F, norm = "max", remHotspot = T, lim.hotspot.int = 0.6) + facet_wrap(vars(fileIndx), ncol = 1) +
    labs(subtitle = paste("m/z:", outData$mz[[3]])) +
    scale_y_continuous()
  plotMSI(dataX = outData$data, indx = outData$indx,mzlist = outData$mz, mz = ions$mz[2], ppm = 300, Title = ions$name[2],
          revBG = F, norm = "max", remHotspot = T, lim.hotspot.int = 0.6) + facet_wrap(vars(fileIndx), ncol = 1)
dev.off()


# # Tissue segmentation -----------------------------------------------------
# # Read tissue segmentation
# tissueSeg <- png::readPNG("./input/SegmentMask.png")
# tissueSegMod <- png::readPNG("./input/Asset 2.png")[,,1:3]
# # colorArr <- c("R", "G", "B")
# 
# tissueSegNew <- tissueSeg[,,1] %>% 
#   # apply(tissueSeg[,,1:3], 1:2, FUN = function(x) {
#   # matchIndx = which(x == max(x))
#   # if(length(matchIndx) > 1)
#   #   NA
#   # else
#   #   colorArr[which(x == max(x))]
#   # }) %>% 
#   t() %>% 
#   data.frame() %>% 
#   pivot_longer(cols = everything(),names_to = "y", values_to = "color") %>% 
#   mutate(color = ifelse(color == 1, NA, color)) %>% 
#   mutate(color = ifelse(color> 0.5, "G", "R")) %>% 
#   mutate(y = str_sub(y, 2) %>% as.numeric()) %>% 
#   mutate(x = rep(1:totalData$dim_xy[[1]][1], each = totalData$dim_xy[[1]][2])) %>% 
#   filter(!is.na(color))
# 
# # Get the updated segmented masks
# tissueSegModNew <- tissueSegMod %>% 
#   # apply(c(1,2), sum) %>%  
#   apply( c(1,2), FUN = function(x) {
#     if(x[3] > 0.8 & sum(x[1:2] < 0.5)) {
#        NA
#     }
#     else if(sum(x) < 1){
#       "R"
#     }
#     else
#       "G"
#   # matchIndx = which(x == max(x))
#   # if(length(matchIndx) > 1)
#   #   NA
#   # else
#   #   colorArr[which(x == max(x))]
#   }) %>% 
#   t() %>% 
#   data.frame() %>% 
#   pivot_longer(cols = everything(),names_to = "y", values_to = "color") %>% 
#   # mutate(color = ifelse(color < 1, "R", "G")) %>% 
#   mutate(y = str_sub(y, 2) %>% as.numeric()) %>% 
#   mutate(x = rep(1:totalData$dim_xy[[1]][1], each = totalData$dim_xy[[1]][2]))
#   # filter(x > 250)
# 
# # Update tissue mask using updated segmentations
# tissueSegNew <- tissueSegModNew %>% full_join(tissueSegNew, by = c("x", "y")) %>%
#   filter(!is.na(color.y)) %>% 
#   filter(!is.na(color.x)) %>% 
#   mutate(color = color.x) %>%
#   # mutate(color = ifelse(is.na(color.x), color.y, color.x)) %>%
#   select(-color.x, -color.y)
##################

# Tissue segmentation -----------------------------------------------------
# Read tissue segmentation
tissueSeg <- png::readPNG("./input/human overlay_MASKS.png")[1:178,1:410,1:3]
tissueSegMod <- png::readPNG("./input/human overlay_BINARY IMAGE.png")[1:178,1:410,1:3]
# colorArr <- c("R", "G", "B")

tissueSegNew <- tissueSeg %>% 
  # apply(c(1,2), sum) %>%  
  apply( c(1,2), FUN = function(x) {
    if(x[3] > 0.8 & sum(x[1:2] < 0.5)) {
      "T"
    }
    else
      NA
    # matchIndx = which(x == max(x))
    # if(length(matchIndx) > 1)
    #   NA
    # else
    #   colorArr[which(x == max(x))]
  }) %>% 
  t() %>% 
  data.frame() %>% 
  pivot_longer(cols = everything(),names_to = "y", values_to = "color") %>% 
  # mutate(color = ifelse(color < 1, "R", "G")) %>% 
  mutate(y = str_sub(y, 2) %>% as.numeric()) %>% 
  mutate(x = rep(1:totalData$dim_xy[[1]][1], each = totalData$dim_xy[[1]][2]))

# tissueSegNew %>% ggplot(aes(x,y,fill=color))+geom_raster()+coord_equal()+scale_y_reverse()

# Get the updated segmented masks
tissueSegModNew <- tissueSegMod %>% 
  # apply(c(1,2), sum) %>%  
  apply( c(1,2), FUN = function(x) {
    if(sum(x) < 1){
      "R"
    }
    else
      "G"
    # matchIndx = which(x == max(x))
    # if(length(matchIndx) > 1)
    #   NA
    # else
    #   colorArr[which(x == max(x))]
  }) %>% 
  t() %>% 
  data.frame() %>% 
  pivot_longer(cols = everything(),names_to = "y", values_to = "color") %>% 
  # mutate(color = ifelse(color < 1, "R", "G")) %>% 
  mutate(y = str_sub(y, 2) %>% as.numeric()) %>% 
  mutate(x = rep(1:totalData$dim_xy[[1]][1], each = totalData$dim_xy[[1]][2]))
# filter(x > 250)

# Update tissue mask using updated segmentations
tissueSegNew <- tissueSegModNew %>% full_join(tissueSegNew, by = c("x", "y")) %>%
  filter(!is.na(color.y)) %>% 
  filter(!is.na(color.x)) %>% 
  mutate(color = color.x) %>%
  # mutate(color = ifelse(is.na(color.x), color.y, color.x)) %>%
  select(-color.x, -color.y)

# pdf(file.path(plots_dir, "Single ions images-new-scaled-overlaidSeg.pdf"),
#     onefile = TRUE,
#     width = 10)
# plotMSI(dataX = outData$data, indx = outData$indx,mzlist = outData$mz, mz = ions$mz[2], ppm = 300, Title = ions$name[2],
#         revBG = F,  norm = "max", remHotspot = T, lim.hotspot.int = 0.6) + facet_wrap(vars(fileIndx), ncol = 1) +
# geom_raster(data = tissueSegNew, aes(x= x, y = y, fill= ifelse(color=="R", 0,1), alpha = 0.2)) 
#   
# dev.off()


# Subset data -------------------------------------------------------------

outData.subSet <- outData
subset.Indx <- outData.subSet$indx %>% 
  mutate(rowN = row_number()) %>% 
  right_join(tissueSegNew) 

outData.subSet$data <- outData.subSet$data[subset.Indx$rowN, ]
outData.subSet$indx <- subset.Indx

outData.subSet$indx$tissue <- map(1:nrow(outData.subSet$indx), function(i) {
  indx = outData.subSet$indx[i,]
  if(indx$x <= 113)
    return(1)
  else if((indx$x > 113 & indx$x <=290) & indx$y>76)
    return(2)
  else if(indx$y <= 77)
    return(3)
  else if(indx$x > 238)
    return(4)
}) %>% unlist()

# pdf(file.path(plots_dir, "Tissue Segments.pdf"),
#     onefile = TRUE,
#     width = 10)
# ggplot(outData.subSet$indx) + 
#   geom_raster(aes(x,y, fill = as.character(tissue))) + 
#   theme_void() +coord_equal() + scale_y_reverse() + 
#   scale_fill_discrete() + labs(fill = "Tissue")
# dev.off()
# 
# pdf(file.path(plots_dir, "Single ions images-new-scaled-overlaidSeg-subset.pdf"),
#     onefile = TRUE,
#     width = 10)
# plotMSI(dataX = outData.subSet$data, indx = outData.subSet$indx,mzlist = outData.subSet$mz, mz = ions$mz[2], ppm = 300, Title = ions$name[2],
#         revBG = F,  norm = "max", remHotspot = T, lim.hotspot.int = 0.6) + facet_wrap(vars(fileIndx), ncol = 1) +
#   geom_raster(data = tissueSegNew, aes(x= x, y = y, fill= ifelse(color=="R", 0,1), alpha = 0.05)) 
# 
# dev.off()

# Proportions plot (Suppl Fig 2j)------------------------------------------

outData.subset.pa = cbind(outData.subSet$indx, list(pa = outData.subSet$data[,2]))

# Export data for Paolo 
outData.subset.pa %>% 
  select(x,y,color, tissue, pa) %>% 
  write_rds("Human_PA_all.rds")
dframe = data.frame(myc = character(0),
                    ratio = numeric(0),
                    id = numeric(0))
for (i in 1:4) {

  pa.response <- outData.subset.pa[outData.subset.pa$tissue == i,]
  pa.response$binary <- pa.response$pa[pa.response$tissue == i] > median(pa.response$pa[pa.response$tissue == i])
  
  subset_ = pa.response$binary[pa.response$color == "G"]
  n_hi = sum(subset_) / length(subset_)
  subset_ = pa.response$binary[pa.response$color == "R"]
  n_lo = sum(subset_) / length(subset_)
  
  dframe = rbind(dframe,
                 data.frame(ratio = c(n_lo, n_hi),
                            myc = c('lo', 'hi'),
                            id = i
                            ))
}
  
# Mean PA intensity per tissue
outData.subset.pa %>% 
  group_by(color, tissue) %>% 
  summarise(Mean.PA = mean(pa)) %>% 
  mutate(Myc = ifelse(color == "G", "hi", "lo")) %>% 
  ungroup() %>% 
  select(Myc, tissue, Mean.PA) %>% 
  write.csv(file.path(csv_dir, "mean_pa_biopsy.csv"), row.names = F)

# # Jaccard index to check the correspondence between PA and Myc labels
# outData.subset.pa = cbind(outData.subSet$indx, list(pa = outData.subSet$data[,2]))
# dframe = data.frame(myc = character(0),
#                     ratio = numeric(0),
#                     id = numeric(0))
# for (i in 1:4) {
#   
#   pa.response <- outData.subset.pa[outData.subset.pa$tissue == i,]
#   pa.response$binary <- pa.response$pa[pa.response$tissue == i] > median(pa.response$pa[pa.response$tissue == i])
#   
#   subset_ = pa.response$binary[pa.response$color == "G"]
#   n_hi = sum(subset_) / length(subset_)
#   subset_ = pa.response$binary[pa.response$color == "R"]
#   n_lo = sum(subset_) / length(subset_)
#   
#   dframe = rbind(dframe,
#                  data.frame(ratio = c(n_lo, n_hi),
#                             myc = c('lo', 'hi'),
#                             id = i
#                  ))
# }

# To test that the proportions of high Myc are larger than 0.5, use a logistic regression
Y <- dframe %>% 
  filter(myc == "hi") %>% 
  pull(ratio)
Model = glm(Y ~ 1, family = quasibinomial())
gtsummary::tbl_regression(Model, exponentiate = TRUE)

library(ggpubr)
pdf(file.path(plots_dir, "PA-proportions.pdf"),
    onefile = TRUE,
    width = 10)
ggerrorplot(
  dframe,
  x = 'myc', y = 'ratio',
  color = 'black',
  add = 'jitter',
  add.params = list(color = 'myc', size=2)
) + scale_color_brewer(palette='Set1') +
  geom_text(aes(label = id), nudge_x = 0.01, nudge_y = 0.01)
dev.off()


pdf(file.path(plots_dir, "Tissue-segmentations.pdf"),
    onefile = TRUE,
    width = 10)
ggplot(outData.subSet$indx, aes(x,y,fill = color))+geom_raster()+coord_equal()+scale_y_reverse()+
  theme_void() +
  theme(
    plot.subtitle = element_text(
      size = 7,
      hjust = 0,
      face = "italic",
      color = "black"
    ),
    panel.grid.major = element_blank(),
    axis.line = element_blank()
  ) +
  scale_fill_manual(values = c("seagreen2", "tomato1"))
dev.off()
