# Miscellaneous functions used in network analysis and various  other plots
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Authors: Paolo Inglese (Imperial College London), Avinash Ghanate (The Francis Crick Institute, London)
#
# library(rhdf5)
# library(EBImage)
# library(sva)
library(scales)
library(tidyverse)
library(viridisLite)
library(igraph)
library(ggraph)

loadMSI = function(dir) {
  
  res = list(X = c(), cmz = c(), dxy = c())
  res$cmz = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'cmz')
  res$dxy = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'dim_xy')
  res$X = h5read(file.path(dir, 'intensity_mat_multi.h5'), 'X')
  h5closeAll()
  
  res$dxy = res$dxy[2:1]
  res$X = t(res$X)
  res$cmz = sort(res$cmz)
  
  return(res)
}

medNorm = function(x) {
  x[x == 0] = NA
  x = x / apply(x, 1, median, na.rm = T)
  x[is.na(x)] = 0
  return(x)
}

listMSIDirs = function(root_dir) {
  dirs = list.dirs(root_dir, full.names = T, recursive = F)
  msi_dirs = c()
  for (i in 1:length(dirs)) {
    if (file.exists(file.path(dirs[i], 'intensity_mat_multi.h5'))) {
      msi_dirs = c(msi_dirs, dirs[i])
    }
  }
  return(msi_dirs)
}

getROI = function(X, dxy, min_size = 10) {
  
  roi = apply(X, 1, function(x) any(x != 0))
  
  lbl_im = bwlabel(matrix(roi, dxy[2:1]))
  ul = unique(lbl_im[lbl_im != 0])
  areas = sapply(ul, function(x) sum(lbl_im == x))
  sel_ul = ul[areas > min_size]
  
  lbl_im = lbl_im %in% sel_ul
  return(matrix(lbl_im, dxy[2:1]))
}

isoProb = function(N, m) {
  prob_iso = choose(N, m) * (0.01109^m * 0.98891^(N - m))
  return(prob_iso)
}

searchMetab = function(tab, cmz, intensity, adduct, delta, max_err = 30) {
  # Search metabolite
  matches = data.frame(name=character(0),
                       mz=numeric(0),
                       M=numeric(0),
                       idx=numeric(0),
                       error=numeric(0),
                       ncarb=numeric(0))
  
  for (i in 1:nrow(tab)) {
    m = tab$mz[i] + adduct
    
    for (iso in c(0:tab$C[i])) {
      mz_th = m + iso * delta
      ppm_err = (cmz - mz_th) / mz_th * 1e6
      idx = which(abs(ppm_err) < max_err)
      if (length(idx) == 0) {
        next()
      } else if (length(idx) > 1) {
        cat('found', length(idx), 'matches\n')
        idx = idx[which.max(apply(intensity[, idx], 2, mean))]
      }
      err = (cmz[idx] - mz_th) / mz_th * 1e6
      if (abs(err) < max_err) {
        matches = rbind(matches, data.frame(name=tab$name[i],
                                            mz=tab$mz[i],
                                            M=iso,
                                            mz_th=mz_th,
                                            mz_obs=cmz[idx],
                                            idx=idx,
                                            error=err,
                                            C=tab$C[i],
                                            labelled=tab$label.possible[i]))
      }
    }
  }
  stopifnot(!any(duplicated(matches$idx)))
  return(matches)
}

correctLabelled = function(X, matches) {
  n_ = unique(matches$name)
  for (i in 1:length(n_)) {
    match_name = subset(matches, name == n_[i])
    print(match_name)
    
    if (match_name$label[1] == 'N') {
      next()
    }
    
    if (sum(match_name$M == 0) == 0) 
      next()
    
    idx0 = subset(match_name, M == 0)$idx
    y0 = X[, idx0]
    
    if (nrow(match_name) != 1) {
      for (j in 1:nrow(match_name)) {
        M = match_name$M[j]
        C = match_name$C[j]
        if (M > 0) {
          p = isoProb(C, M)
          thr = y0 * p / (1 - p)
          print(max(thr))
          curr.idx = matches$idx[(matches$name == n_[i]) & (matches$M == M)]
          curr.vals = X[, curr.idx]
          curr.vals = curr.vals - thr
          curr.vals[curr.vals < 0.0] = 0
          X[, curr.idx] = curr.vals
        }
      }
    }
  }
  return(X)
}

computeME = function(msi_data, tissue_labels, modules) {
  
  umod = sort(unique(modules))
  nmodules = length(umod)
  n_runs = length(msi_data)
  
  # Merge data from all runs
  x_all = c()
  batch = c()
  n_pix = c()
  y_all = c()
  for (i in 1:n_runs) {
    x_all = rbind(x_all, msi_data[[i]]$X)
    n_pix = c(n_pix, nrow(msi_data[[i]]$X))
    y_all = c(y_all, tissue_labels[[i]][!is.na(tissue_labels[[i]])])
    batch = c(batch, rep(i, nrow(msi_data[[i]]$X)))
  }
  
  idx_train = which(y_all %in% c('lo', 'hi'))
  idx_pred = which(y_all == 'mix')
  x_train = x_all[idx_train, ]
  x_pred = x_all[idx_pred, ]
  
  mu = apply(x_train, 2, mean)
  sig = apply(x_train, 2, sd)
  
  x_train = scale(x_train, mu, sig)
  x_pred = scale(x_pred, mu, sig)
  
  ME_all =  matrix(NA, nrow(x_all), nmodules,
                   dimnames = list(NULL, paste0('ME', umod)))
  for (m in umod) {
    pca = gmodels::fast.prcomp(x_train[, modules == m],
                               center = F, scale. = F)
    if (cor(pca$x[, 1], apply(x_train[, modules == m], 1, mean)) < 0) {
      pca$x[, 1] = -pca$x[, 1]
      pca$rotation = -pca$rotation
    }
    
    ME_all[idx_train, colnames(ME_all) == paste0('ME', m)] = pca$x[, 1]
    ME_all[idx_pred, colnames(ME_all) == paste0('ME', m)] = 
      x_pred[, modules == m] %*% pca$rotation[, 1]
  }
  
  ME = vector('list', n_runs)
  offs = 0
  for (i in 1:n_runs) {
    ME[[i]]$eigengenes = matrix(NA, prod(msi_data[[i]]$dxy), nmodules,
                                dimnames = list(NULL, colnames(ME_all)))
    ME[[i]]$eigengenes[!is.na(tissue_labels[[i]]), ] = ME_all[(offs+1):(offs+n_pix[i]), ]
    offs = offs + n_pix[i]
  }
  
  return(ME)
}


fixBatch = function(msi_data) {
  
  n_runs = length(msi_data)
  
  # Merge data and correct batch effect
  x_all = c()
  batch = c()
  n_pix = c()
  for (i in 1:n_runs) {
    x_all = rbind(x_all, msi_data[[i]]$X)
    n_pix = c(n_pix, nrow(msi_data[[i]]$X))
    batch = c(batch, rep(i, nrow(msi_data[[i]]$X)))
  }
  
  x_all = t(sva::ComBat(t(x_all), batch = factor(batch)))
  
  offs = 0
  for (i in 1:n_runs) {
    msi_data[[i]]$X = x_all[(offs + 1):(offs + n_pix[i]), ]
    offs = offs + n_pix[i]
  }
  
  return(msi_data)
}

# Normalization function --------------------------------------------------

normData <- function(data, norm = "default")
{
  normVal = switch (
    norm,
    default = 1,
    max = apply(data, 2, max, na.rm = T),
    median = apply(data, 1, function(x){median(x[x>0], na.rm = T)})
  )
  
  if(norm != 'max')
    data <- data/normVal
  else if(norm == 'max')
    data <- t(t(data)/normVal)
}

# Plot MSI function -------------------------------------------------------

plotMSI <- function(dataX, mzlist = NULL, indx = NULL, mz = NULL, ppm = 10, ticData = NULL, Title = '', norm = 'default', AvgIC = FALSE, usePlotly = FALSE, colorScale = "Magma", revBG = TRUE, remHotspot = TRUE, lim.hotspot = 0.999, lim.hotspot.int = NULL)
{
  ## Normalization methods:
  ## norm = 
  ## default = No normalization,
  # max = Max intensity normalization
  # median = Median intensity normalization
  
  
  if(is.null(indx))
  {
    print('Mask indexes not received. Provide "indx".')
    return()
  }
  
  # No. of m/z in datacube
  mzNum <- length(mzlist)
  
  dataX <- as.data.frame(dataX)
  
  # Get m/z index for requested metabolic ion m/z
  mzI <- matrix(NA, nrow = length(mz), ncol = 1)
  for(i in 1:length(mz))
  {    
    tmp_mzI <-
      which(abs(mzlist - mz[i]) * 1e6 < mz[i] * ppm)         ## within PPM window search
    
    if (length(tmp_mzI) > 1)
      tmp_mzI = tmp_mzI[which.min(abs(mz[i] - mzlist[tmp_mzI]))]        ## Find m/z with lowest PPM difference
    
    if(length(tmp_mzI) > 0)
      mzI[i] <- tmp_mzI
  }
  Title <- ifelse(Title == "", rep("", times = length(mz)), Title)
  
  
  if (sum(!is.na(mzI)) == 0)
  {
    message('m/z not detected!')
    return()
  }
  
  for(fID in unique(indx$fileN))
  {
    dataX[indx$fileN %in% fID, 1:mzNum] <- normData(dataX[indx$fileN %in% fID, 1:mzNum], norm)
  }
  
  
  # If more than one m/z requested
  gList <- vector(mode = "list", length = length(mzI))
  foundMZIndx <- which(!is.na(mzI))
  
  gList <- lapply(foundMZIndx, function(i) plotSI(dataX[, mzI[i]], indx = indx, mz = mzlist[mzI[i]], query.mz = mz[i], Title = Title[i], usePlotly = usePlotly, colorScale = colorScale, revBG = revBG, remHotspot = remHotspot, lim.hotspot = lim.hotspot, lim.hotspot.int = lim.hotspot.int))
  
  if(sum(!is.na(mzI)) > 1)
    return(gList)
  else
    return(gList[[1]])
}


# Plot function for MSI single ion image ----------------------------------

plotSI <- function(z.fill = NULL, indx = NULL, query.mz = NULL, mz = NULL, Title="", usePlotly = FALSE, colorScale = "Magma", revBG = TRUE, remHotspot = TRUE, lim.hotspot = 0.999, lim.hotspot.int = NULL)
{
  
  if(is.null(z.fill) | is.null(indx) | is.null(mz))
  {
    print("Important input variables are missing. Provide z.fill,  indx, query.mz and mz!")
    return(NULL)
  }
  
  # ColorScale
  colOpt <- switch (colorScale,
                    default = "D",
                    Viridis = "D",
                    Magma = "A",
                    Inferno = "B",
                    Plasma = "C",
                    Cividis = "E"
  )
  if(remHotspot)
  {
    max.int <- max(z.fill, na.rm = T)
    if(is.null(lim.hotspot.int))
      limit.int <- quantile(z.fill, lim.hotspot , na.rm = T)[[1]] 
    else
      limit.int <- lim.hotspot.int
    colPalette <- viridis(round((limit.int/max.int)*256, 0), direction = 1, option = colOpt)
    colPalette <- c(colPalette, rep(colPalette[length(colPalette)], 256 - round((limit.int/max.int)*256, 0)))
  }
  else
  {
    colPalette <- viridis(256, direction = 1, option = colOpt)
  }
  
  g <- ggplot() +
    labs(x = '', y = '', fill = '') +
    scale_y_reverse() +
    coord_equal() +
    theme_void() +
    theme(
      plot.subtitle = element_text(
        size = 7,
        hjust = 0,
        face = "italic",
        color = "black"
      ),
      panel.grid.major = element_blank(),
      axis.line = element_blank()
    ) +
    geom_raster(data = indx, aes(x = x, y = y, fill = z.fill), interpolate = TRUE) +
    scale_fill_gradientn(colours = colPalette) +
    ggtitle(Title) +
    labs(subtitle = paste('m/z:', round(mz, 4), ' PPM: ', round(abs(mz - query.mz) * 1e6 / query.mz,4))) 
  
  if(revBG)
    g <- g + theme(panel.background = element_rect(fill = rgb(0, 0, 0, 1)))
  
  # Compile all images in a list
  return(g)

}

# Circos plot function ----------------------------------------------------

edgeGraph <- function(ions.cor = NULL, mod.mz = NULL, mod.membership = NULL, mz.class= NULL)
{
  
  ions.cor[lower.tri(ions.cor, diag = T)] <- 0
  # create a dataframe with connection between leaves (individuals)
  allConnDF <- data.frame(from = "orig", to = format(mod.mz, nsmall = 4))
  
  # create a vertices data.frame. One line per object of our hierarchy
  vertices <- data.frame(
    name = c("orig",format(mod.mz, nsmall = 4)),
    value = c(NA,mod.membership)
  )
  
  # The connection object must refer to the ids of the leaves:
  connDF <- which(ions.cor > 0, T) %>% cbind(ions.cor[.]) %>% as.data.frame() %>% rename(from = row, to = col, value = V3) %>% mutate(from = from + 1, to = to + 1)
  mygraph <- graph_from_data_frame( allConnDF, vertices = vertices)
  
  if(length(connDF) == 0)
    return()
  mz.class <- c(NA, mz.class)
  ggraph(mygraph, layout = 'dendrogram', circular = TRUE) + 
    geom_conn_bundle(data = get_con(from = connDF$from, to = connDF$to), edge_alpha=0.4,  width=0.8, tension = 1, aes(colour = rep(connDF$value,each =3))) +
    scale_edge_color_viridis(begin = 0.15, end = 0.9,discrete = FALSE, option = "C", name = "Correlation", direction = -1)+
    geom_node_text(aes(x = x*1.25, y=y*1.25, filter = leaf, label=name, color = mz.class,
                       angle = ifelse(270>node_angle(x, y) & node_angle(x, y)>90, node_angle(x, y)+180,node_angle(x, y))), 
                   size=4, alpha=1, show.legend = F) +
    geom_node_point(aes(filter = leaf, x = x*1.07, y=y*1.07, size=value, color = mz.class, alpha=0.2), show.legend = T) +
    scale_size_continuous(range = c(0.3,6)) +
    labs(color = "Metabolite class", size = "Module membership") +
    theme_void()+guides(alpha = FALSE)  +
    coord_equal() + 
    theme(
      plot.margin=unit(c(0,0,0,0),"cm"),
    ) +
    expand_limits(x = c(-1.35, 1.35), y = c(-1.35, 1.35))
}